@@ -134,7 +134,7 @@ class PSBTView:
             num_inputs, num_outputs,
             offset, first_scope,
             version=None, tx_offset=None,
-            compress=False,
+            compress=CompressMode.KEEP_ALL,
         ):
         if version != 2 and tx_offset is None:
             raise PSBTError("Global tx is not found, but PSBT version is %d" % version)
@@ -162,7 +162,7 @@ class PSBTView:
         self._hash_script_pubkeys = None
 
     @classmethod
-    def view(cls, stream, offset=None, compress=False):
+    def view(cls, stream, offset=None, compress=CompressMode.KEEP_ALL):
         if offset is None and hasattr(stream, 'tell'):
             offset = stream.tell()
         offset = offset or 0
@@ -598,23 +598,36 @@ class PSBTView:
                     counter = 1
             # if we use HDKey
             else:
-                # TODO: add taproot derivation paths and scripts
+                bip32_derivations = []
+                for pub in inp.taproot_bip32_derivations:
+                    leaf_hashes, derivation = inp.taproot_bip32_derivations[pub]
+                    if derivation.fingerprint == fingerprint:
+                        bip32_derivations.append((pub, derivation))
+
+                # "Legacy" support for workaround when BIP-371 Taproot psbt fields aren't available
                 for pub in inp.bip32_derivations:
-                    # check if it is root key
-                    if inp.bip32_derivations[pub].fingerprint == fingerprint:
-                        hdkey = root.derive(inp.bip32_derivations[pub].derivation)
-                        mypub = hdkey.key.get_public_key()
-                        if mypub != pub:
-                            raise PSBTError("Derivation path doesn't look right")
-                        pk = hdkey.taproot_tweak(b"")
-                        if pk.xonly() in sc.data:
-                            sig = pk.schnorr_sign(h)
-                            # sig plus sighash flag
-                            wit = sig.serialize()
-                            if inp_sighash != SIGHASH.DEFAULT:
-                                wit += bytes([inp_sighash])
-                            inp.final_scriptwitness = Witness([wit])
-                            counter = 1
+                    derivation = inp.bip32_derivations[pub]
+                    if derivation.fingerprint == fingerprint:
+                        bip32_derivations.append((pub, derivation))
+
+                for pub, derivation in bip32_derivations:
+                    hdkey = root.derive(derivation.derivation)
+
+                    # Taproot BIP32 derivations use X-only pubkeys
+                    xonly_pub = hdkey.key.xonly()
+                    mypub = ec.PublicKey.from_xonly(xonly_pub)
+
+                    if mypub != pub:
+                        raise PSBTError("Derivation path doesn't look right")
+                    pk = hdkey.taproot_tweak(b"")
+                    if pk.xonly() in sc.data:
+                        sig = pk.schnorr_sign(h)
+                        # sig plus sighash flag
+                        wit = sig.serialize()
+                        if inp_sighash != SIGHASH.DEFAULT:
+                            wit += bytes([inp_sighash])
+                        inp.final_scriptwitness = Witness([wit])
+                        counter = 1
             if counter:
                 ser_string(sig_stream, b"\x08")
                 ser_string(sig_stream, inp.final_scriptwitness.serialize())
@@ -662,7 +675,7 @@ class PSBTView:
             sig_stream.write(b"\x00")
         return counter
 
-    def write_to(self, writable_stream, compress=False,
+    def write_to(self, writable_stream, compress=None,
             extra_input_streams=[],
             extra_output_streams=[],
     ):
@@ -676,6 +689,9 @@ class PSBTView:
         For psbtv0 it will have global tx and partial sigs for all inputs
         For psbtv2 it will have version, tx_version, locktime, per-vin data, per-vout data and partial sigs
         """
+        if compress is None:
+            compress = self.compress
+
         # first we write global scope
         self.stream.seek(self.offset)
         res = read_write(self.stream, writable_stream, self.first_scope-self.offset)
@@ -688,7 +704,7 @@ class PSBTView:
                 extra = InputScope.read_from(s)
                 inp.update(extra)
             if compress:
-                inp.clear_metadata()
+                inp.clear_metadata(compress=compress)
             res += inp.write_to(writable_stream, version=self.version)
 
         # write all outputs
@@ -699,7 +715,7 @@ class PSBTView:
                 extra = OutputScope.read_from(s)
                 out.update(extra)
             if compress:
-                out.clear_metadata()
+                out.clear_metadata(compress=compress)
             res += out.write_to(writable_stream, version=self.version)
 
         return res
